#include <stdlib.h>

#include <string>
#include "tstring.h"
#include "tbytevector.h"

#ifndef HAVE_LIBRCC
# include <config.h>
#endif

#ifdef HAVE_LIBRCC
# include <librcc.h>
# include <string.h>
#endif /* HAVE_LIBRCC */


#ifdef HAVE_LIBRCC
# define ID3_CLASS 0
# define ID3V2_CLASS 1
# define UTF_CLASS 2
# define OUT_CLASS 3
static rcc_class classes[] = {
    { "id3", RCC_CLASS_STANDARD, NULL, NULL, "ID3 Encoding", 0 },
    { "id3v2", RCC_CLASS_STANDARD, "id3", NULL, "ID3 v.2 Encoding", 0 },
    { "utf", RCC_CLASS_KNOWN, "UTF-8", NULL, "Unicode Encoding", 0},
    { "out", RCC_CLASS_TRANSLATE_LOCALE, "LC_CTYPE", NULL, "Output Encoding", 0 },
    { NULL, RCC_CLASS_STANDARD, NULL, NULL, NULL, 0 }
};

static int rcc_initialized = 0;

static rcc_context ctx = NULL;
#endif /* HAVE_LIBRCC */


void rccPatchFree() {
#ifdef HAVE_LIBRCC
    if (rcc_initialized) {
       rccFree();
       rcc_initialized = 0;
    }
#endif /* HAVE_LIBRCC */
}

void rccPatchInit() {
#ifdef HAVE_LIBRCC
    if (rcc_initialized) return;
    rccInit();
    rccInitDefaultContext(NULL, 0, 0, classes, 0);
    rccLoad(NULL, "xmms");
    rccInitDb4(NULL, NULL, 0);
    rcc_initialized = 1;
#endif /* HAVE_LIBRCC */
}

void rccPatchSetContext(void *newctx) {
#ifdef HAVE_LIBRCC
    if (newctx) {
	ctx = (rcc_context)newctx;
	rcc_initialized = 1;
    }
#endif /* HAVE_LIBRCC */
}

static void rccPatchTryInit() {
#ifdef HAVE_LIBRCC
    if (!rcc_initialized) {
	rccPatchInit();
	if (rcc_initialized) atexit(rccPatchFree);	
    }
#endif /* HAVE_LIBRCC */
}


TagLib::ByteVector rccPatchRecodeOutput(const std::string &s) {
    TagLib::ByteVector v;
#ifdef HAVE_LIBRCC
    size_t rlen;
    char *res;

    rccPatchTryInit();

    res = rccSizedRecode(ctx, UTF_CLASS, OUT_CLASS, s.c_str(), s.length(), &rlen);
    if (res) v.setData(res, rlen);
    else v.setData(s.c_str(), s.length());

    return v;
#else
    v.setData("", 0);
    
    return v;
#endif /* HAVE_LIBRCC */
}

TagLib::ByteVector rccPatchRecodeOutputID3(const std::string &s, bool v2 = false) {
    TagLib::ByteVector v;
#ifdef HAVE_LIBRCC
    size_t rlen;
    char *res;

    rccPatchTryInit();

    res = rccSizedRecode(ctx, UTF_CLASS, v2?ID3V2_CLASS:ID3_CLASS, s.c_str(), s.length(), &rlen);
    if (res) v.setData(res, rlen);
    else v.setData(s.c_str(), s.length());

    return v;
#else
    v.setData("", 0);
    
    return v;
#endif /* HAVE_LIBRCC */
}

TagLib::ByteVector rccPatchRecodeInput(const std::string &s) {
    TagLib::ByteVector v;
#ifdef HAVE_LIBRCC
    size_t rlen;
    char *res;

    rccPatchTryInit();

    res = rccSizedRecode(ctx, OUT_CLASS, UTF_CLASS, s.c_str(), s.length(), &rlen);
    if (res) v.setData(res, rlen);
    else
#endif /* HAVE_LIBRCC */
    v.setData("", 0);
    
    return v;
}

TagLib::ByteVector rccPatchRecodeInputID3(const std::string &s, bool v2 = false) {
    TagLib::ByteVector v;
#ifdef HAVE_LIBRCC
    size_t rlen;
    char *res;

    rccPatchTryInit();

    res = rccSizedRecode(ctx, v2?ID3V2_CLASS:ID3_CLASS, UTF_CLASS, s.c_str(), s.length(), &rlen);
    if (res) v.setData(res, rlen);
    else
#endif /* HAVE_LIBRCC */
    v.setData("", 0);

    return v;
}

TagLib::String::Type rccPatchGetLocaleType() {
#ifdef HAVE_LIBRCC
    size_t len;
    char charset[32];

    rccPatchTryInit();
    
    if (!rccLocaleGetCharset(charset, NULL, 31)) {
	if (!strncmp(charset, "UTF", 3)) {
	    len = strlen(charset);
	    
	    if (charset[len-1]=='8') return TagLib::String::UTF8;
	    if (!strcmp(charset+(len-2),"16")) return TagLib::String::UTF16;
	    if (!strcmp(charset+(len-4),"16LE")) return TagLib::String::UTF16LE;
	    if (!strcmp(charset+(len-4),"16BE")) return TagLib::String::UTF16BE;
	}
	return TagLib::String::Latin1;
    }
#endif /* HAVE_LIBRCC */
    return TagLib::String::UTF8;
}

TagLib::String::Type rccPatchGetID3Type() {
#ifdef HAVE_LIBRCC
    size_t len;
    const char *charset;

    rccPatchTryInit();
    
    charset = rccGetCurrentCharsetName(ctx, ID3V2_CLASS);
    if (charset) {
	if (!strncmp(charset, "UTF", 3)) {
	    len = strlen(charset);
	    
	    if (charset[len-1]=='8') return TagLib::String::UTF8;
	    if (!strcmp(charset+(len-2),"16")) return TagLib::String::UTF16;
	    if (!strcmp(charset+(len-4),"16LE")) return TagLib::String::UTF16LE;
	    if (!strcmp(charset+(len-4),"16BE")) return TagLib::String::UTF16BE;
	}
	return TagLib::String::Latin1ID3V2;
    }
#endif /* HAVE_LIBRCC */
    return TagLib::String::Latin1;
}
